package com.loafle.overflow.container.configuration;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Arrays;
import java.util.List;

import com.loafle.commons.server.Server;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.ServerChannel;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;

/**
 * ContainerConfiguration
 */
@Configuration
public class ContainerConfiguration {
  @Autowired
  private List<ChannelHandler> pipelineChannelHandlers;

  @Bean
  public List<Class<? extends ServerChannel>> channelClasses() {
    return Arrays.asList(NioServerSocketChannel.class);
  }

  @Bean
  public List<Server.ChannelOptionItem<?>> channelOptions() {
    return Arrays.asList(new Server.ChannelOptionItem<>(ChannelOption.SO_BACKLOG, 100));
  }

  @Bean
  public List<ChannelHandler> channelHandlers() {
    return Arrays.asList(new LoggingHandler(LogLevel.INFO));
  }

  @Bean
  public ChannelInitializer<?> channelInitializer() {
    return new ChannelInitializer<SocketChannel>() {
          @Override
          public void initChannel(SocketChannel ch) throws Exception {
            ChannelPipeline cp = ch.pipeline();
            for (ChannelHandler channelHandler : pipelineChannelHandlers) {
              cp.addLast(channelHandler);
            }
          }
      };
  }

  @Bean
  public SocketAddress address() {
    return new InetSocketAddress("127.0.0.1", 60000);
  }
}